WHITE = (1, 1, 1)
LIGHT_GREY = (0.75, 0.75, 0.75)
GREY = (0.5, 0.5, 0.5)
DARK_GREY = (0.25, 0.25, 0.25)
BLACK = (0, 0, 0)

RED_PASTEL = (1, 0.75, 0.75)
LIGHT_RED = (1, 0.5, 0.5)
RED = (1, 0, 0)
DARK_RED = (0.5, 0, 0)
VERY_DARK_RED = (0.25, 0, 0)

GREEN_PASTEL = (0.75, 1, 0.75)
LIGHT_GREEN = (0.5, 1, 0.5)
GREEN = (0, 1, 0)
DARK_GREEN = (0, 0.5, 0)
VERY_DARK_GREEN = (0, 0.25, 0)

BLUE_PASTEL = (0.75, 0.75, 1)
LIGHT_BLUE = (0.5, 0.5, 1)
BLUE = (0, 0, 1)
DARK_BLUE = (0, 0, 0.5)
VERY_DARK_BLUE = (0, 0, 0.25)

YELLOW_PASTEL = (1, 1, 0.75)
LIGHT_YELLOW = (1, 1, 0.5)
YELLOW = (1, 1, 0)
DARK_YELLOW = (0.5, 0.5, 0)
VERY_DARK_YELLOW = (0.25, 0.25, 0)

MAGENTA_PASTEL = (1, 0.75, 1)
LIGHT_MAGENTA = (1, 0.5, 1)
MAGENTA = (1, 0, 1)
DARK_MAGENTA = (0.5, 0, 0.5)
VERY_DARK_MAGENTA = (0.25, 0, 0.25)

CYAN_PASTEL = (0.75, 1, 1)
LIGHT_CYAN = (0.5, 1, 1)
CYAN = (0, 1, 1)
DARK_CYAN = (0, 0.5, 0.5)
VERY_DARK_CYAN = (0, 0.25, 0.25)

ORANGE_PASTEL = (1, 0.825, 0.2)
LIGHT_ORANGE = (1, 0.75, 0.1)
ORANGE = (1, 0.5, 0)
DARK_ORANGE = (0.5, 0.25, 0)
VERY_DARK_ORANGE = (0.25, 0.125, 0)

PURPLE_PASTEL = (0.825, 0.2, 1)
LIGHT_PURPLE = (0.75, 0.1, 1)
PURPLE = (0.5, 0, 1)
DARK_PURPLE = (0.25, 0, 0.5)
VERY_DARK_PURPLE = (0.125, 0, 0.25)

AZURE_PASTEL = (0.2, 0.825, 1)
LIGHT_AZURE = (0.1, 0.75, 1.0)
AZURE = (0, 0.5, 1)
DARK_AZURE = (0, 0.25, 0.5)
VERY_DARK_AZURE = (0, 0.125, 0.25)

SPRING_GREEN_PASTEL = (0.2, 1, 0.85)
LIGHT_SPRING_GREEN = (0.1, 1, 0.75)
SPRING_GREEN = (0, 1, 0.5)
DARK_SPRING_GREEN = (0, 0.5, 0.25)
VERY_DARK_SPRING_GREEN = (0, 0.25, 0.125)

PALETTE = [
    [WHITE, LIGHT_GREY, GREY, DARK_GREY, BLACK],
    [RED_PASTEL, LIGHT_RED, RED, DARK_RED, VERY_DARK_RED],
    [MAGENTA_PASTEL, LIGHT_MAGENTA, MAGENTA, DARK_MAGENTA, VERY_DARK_MAGENTA],
    [PURPLE_PASTEL, LIGHT_PURPLE, PURPLE, DARK_PURPLE, VERY_DARK_PURPLE],
    [BLUE_PASTEL, LIGHT_BLUE, BLUE, DARK_BLUE, VERY_DARK_BLUE],
    [AZURE_PASTEL, LIGHT_AZURE, AZURE, DARK_AZURE, VERY_DARK_AZURE],
    [CYAN_PASTEL, LIGHT_CYAN, CYAN, DARK_CYAN, VERY_DARK_CYAN],
    [SPRING_GREEN_PASTEL, LIGHT_SPRING_GREEN, SPRING_GREEN, DARK_SPRING_GREEN, VERY_DARK_SPRING_GREEN],
    [GREEN_PASTEL, LIGHT_GREEN, GREEN, DARK_GREEN, VERY_DARK_GREEN],
    [YELLOW_PASTEL, LIGHT_YELLOW, YELLOW, DARK_YELLOW, VERY_DARK_YELLOW],
    [ORANGE_PASTEL, LIGHT_ORANGE, ORANGE, DARK_ORANGE, VERY_DARK_ORANGE],
]
